#!/usr/bin/env python3
"""
tests automatiques de l'examen
"""
from itertools import zip_longest
from liste import liste_test, Liste, Cellule
# j'ai besoin de rattraper toutes les exceptions possibles
#pylint: disable=bare-except

def liste_test2():
    """
    cree manuellement la liste 0, 6, 12, 15, 18.
    """
    liste = Liste()
    cel6, cel12, cel15, cel18 = [Cellule(v) for v in (6, 12, 15, 18)]
    liste.tete.suivants[0] = cel6
    liste.tete.suivants[1] = cel6
    cel6.suivants[0] = cel12
    cel6.suivants[1] = cel12
    cel12.suivants[0] = cel18
    cel12.suivants[1] = cel15
    cel15.suivants[1] = cel18
    return liste

def contient(cellule, valeurs):
    """
    verifie si la liste contient bien toutes les valeurs donnees, sans cellules en double.
    """
    def contient_rec(cellule, valeurs, cellules):
        """
        on verifie recursivement que les valeurs sont ok, en enregistrant au passage
        l'id des cellules rencontrees.
        """
        if cellule is None:
            return
        cellules.add(id(cellule))

        valeur_courante = cellule.valeur
        position = valeurs.index(valeur_courante)
        valeurs_restantes = valeurs[position+1:]

        diviseurs = (2, 3)
        suivants = [next((v for v in valeurs_restantes if not v % i), None) for i in diviseurs]
        for index, (suivant, diviseur) in enumerate(zip(suivants, diviseurs)):
            if valeur_courante % diviseur:
                assert cellule.suivants[index] is None
            else:
                if suivant is None:
                    assert cellule.suivants[index] is None
                else:
                    assert cellule.suivants[index].valeur == suivant

        contient_rec(cellule.suivants[0], valeurs_restantes, cellules)
        contient_rec(cellule.suivants[1], valeurs_restantes, cellules)

    cellules = set()
    contient_rec(cellule, valeurs, cellules)
    assert len(cellules) == len(valeurs)
    return True

def test_cellules(cellules, valeurs):
    """
    verifie que l'iterateur donne itere bien sur des cellules contenant les valeurs donnees
    """
    for cellule, valeur in zip_longest(cellules, valeurs, fillvalue=-1):
        assert cellule.valeur == valeur

def test_iterateur():
    """
    teste l'iterateur sur les cellules
    """
    try:
        liste = liste_test()
        test_cellules(liste.cellules(0), [0, 2, 6, 8])
        test_cellules(liste.cellules(1), [0, 3, 6])
    except:
        print("la methode 'cellules' ne fonctionne pas")
        return 0
    print("cellules: OK")
    return 1

def test_peupler():
    """
    teste la methode peuplant une liste a partir d'un iterable.
    """
    try:
        assert contient(Liste().peupler([3, 6, 8, 10, 12]).tete, [0, 3, 6, 8, 10, 12])
        assert contient(Liste().peupler([2, 4]).tete, [0, 2, 4])
    except:
        print("la methode 'peupler' ne fonctionne pas")
        return 0
    print("peupler: OK")
    return 3

def test_insertion():
    """
    teste la methode inserant un nouvel element.
    """
    try:
        assert contient(liste_test().insertion(9).tete, [0, 2, 3, 6, 8, 9])
        assert contient(liste_test().insertion(4).tete, [0, 2, 3, 4, 6, 8])
        assert contient(Liste().insertion(3).insertion(2).insertion(6).tete, [0, 2, 3, 6])
        assert contient(Liste().insertion(6).insertion(2).insertion(3).tete, [0, 2, 3, 6])
        assert contient(Liste().insertion(6).insertion(12).insertion(8).tete, [0, 6, 8, 12])
    except:
        print("la methode 'insertion' ne fonctionne pas")
        return 0
    print("insertion: OK")
    return 3

def test_parcours():
    """
    teste l'iterateur sur toutes les cellules.
    """
    try:
        assert [c.valeur for c in liste_test().parcours()] == [0, 2, 3, 6, 8]
        assert [c.valeur for c in liste_test2().parcours()] == [0, 6, 12, 15, 18]
    except:
        print("la methode 'parcours' ne fonctionne pas")
        return 0
    print("parcours: OK")
    return 5

def test_supprimer():
    """
    teste la methode supprimant des cellules.
    """
    try:
        liste = liste_test()
        liste.suppression(8)
        assert contient(liste.tete, [0, 2, 3, 6])
        liste.suppression(2)
        assert contient(liste.tete, [0, 3, 6])
        liste.suppression(3)
        assert contient(liste.tete, [0, 6])

        liste = liste_test2()
        liste.suppression(15)
        assert contient(liste.tete, [0, 6, 12, 18])
        liste.suppression(12)
        assert contient(liste.tete, [0, 6, 18])
    except:
        print("la methode 'suppression' ne fonctionne pas")
        return 0
    print("suppression: OK")
    return 2

def exceptions_peupler():
    """
    verifie si les preconditions de 'peupler' sont testees.
    """
    nombre_tests_ok = 0
    try:
        liste_test().peupler([20])
    except:
        nombre_tests_ok += 1
    try:
        Liste().peupler([20, 18])
    except:
        nombre_tests_ok += 1

    try:
        Liste().peupler([20, 21, 22, 23])
    except:
        nombre_tests_ok += 1

    try:
        Liste().peupler([21, 22, 22])
    except:
        nombre_tests_ok += 1

    if nombre_tests_ok == 4:
        return True
    else:
        print("la methode peupler ne leve pas d'exceptions pour ses pre-conditions")
        return False

def exceptions_insertion():
    """
    verifie si les preconditions de 'insertion' sont testees.
    """
    nombre_tests_ok = 0
    try:
        Liste().insertion(11)
    except:
        nombre_tests_ok += 1

    try:
        liste_test().insertion(6)
    except:
        nombre_tests_ok += 1

    if nombre_tests_ok == 2:
        return True
    else:
        print("la methode insertion ne leve pas d'exceptions pour ses pre-conditions")
        return False

def exceptions_suppression():
    """
    verifie si les preconditions de 'suppression' sont testees.
    """
    nombre_tests_ok = 0
    try:
        Liste().suppression(3)
    except:
        nombre_tests_ok += 1

    try:
        liste_test().suppression(0)
    except:
        nombre_tests_ok += 1

    if nombre_tests_ok == 2:
        return True
    else:
        print("la methode suppression ne leve pas d'exceptions pour ses pre-conditions")
        return False

def test_exceptions():
    """
    verifie si des preconditions non verifiees levent des exceptions.
    """
    if exceptions_peupler() and exceptions_insertion() and exceptions_suppression():
        print("exceptions: OK")
        return 1
    return 0

def main():
    """
    on teste fortement chaque fonction demandee et on calcule votre note.
    """
    note = 0
    assert contient(liste_test().tete, [0, 2, 3, 6, 8])  # on teste le test
    assert contient(liste_test2().tete, [0, 6, 12, 15, 18])  # on teste encore le test
    note += test_iterateur()
    note += test_peupler()
    note += test_insertion()
    note += test_parcours()
    note += test_supprimer()
    note += test_exceptions()
    print("vous avez (a priori):", note, "/ 15")

main()
